local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- Implement a dummy TTD function if we do not get one from another module
if type(s.TimeToDie) ~= 'function'  then
	a.print(L["No TimeToDie function found, using a dummy function. Consider installing Pusikas' Spellflash: TTD."])
	s.TimeToDie = function(unit)
		return 15
	end
end

local CurrentTimestamp = 0
local event, sourceGUID, destGUID, spellId, spellName, destFlags
local myself, mypet
a.AoETargets = {}
local target_is_outsider

a.COMBAT_LOG_EVENT_UNFILTERED = function (event, ...)
	CurrentTimestamp, event, _, sourceGUID, _, _, _, destGUID, _, destFlags, _, spellId, spellName = ...
	target_is_outsider = bit.band(destFlags, COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) > 0
	myself = UnitGUID("player")
	mypet = UnitGUID("pet")
	if sourceGUID == myself and destGUID ~= myself and target_is_outsider and (string.find(event, "_DAMAGE") or string.find(event, "_MISSED")) then
		a.AoETargets[destGUID] = CurrentTimestamp
	end
	if sourceGUID == mypet and destGUID ~= myself and target_is_outsider and (string.find(event, "_DAMAGE") or string.find(event, "_MISSED")) then
		a.AoETargets[destGUID] = CurrentTimestamp
	end
	if event == "UNIT_DIED" or event == "UNIT_DESTROYED" then
		a.AoETargets[destGUID] = nil
	end
	if sourceGUID == myself and a.PSFWatcherStarted and (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and spellId == 53220--[[Steady Focus]] then
		a.BuffPreSteadyFocus = false
	end
end
s.RegisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED", a.COMBAT_LOG_EVENT_UNFILTERED)

-- Check if we are in AoE mode
function a.ActiveEnemies()
	local now = time()
	local i = 0
	foreach(a.AoETargets, function(k, v)
		if v and now - v > 3 then
			a.AoETargets[k] = nil
		else
			i = i + 1
		end
	end)
	return i
end
		
-- Handle Pre-Steady-Shot detection for MM hunters
a.BuffPreSteadyFocus = false
a.PSFWatcherStarted = false

local ResetSteadyFocus = {
	53209--[[Chimera Shot]], 
	19434--[[Aimed Shot]], 
	3044--[[Arcane Shot]],
	5116--[[Concussive Shot]],
	20736--[[Distracting Shot]],
	53351--[[Kill Shot]],
	2643--[[Multi-Shot]],
	19801--[[Tranquilizing Shot]],
	19386--[[Wyvern Sting]],
	109248--[[Binding Shot]],
	131894--[[A Murder of Crows]],
	117050--[[Glaive Toss]],
	109259--[[Powershot]],
	120360--[[Barrage]],
}

a.PreSteadyFocus = function()
	if a.BuffPreSteadyFocus then
		return not s.Casting(56641--[[Steady Shot]], "player") and not s.Casting(77767--[[Cobra Shot]], "player")
	else
		return s.Casting(56641--[[Steady Shot]], "player") or s.Casting(77767--[[Cobra Shot]], "player")
	end
end

a.stopPreSteadyFocus = function()
	if a.PSFWatcherStarted then
		s.UnregisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED")
		a.PSFWatcherStarted = false
		a.BuffPreSteadyFocus = false
	end
end

a.startPreSteadyFocus = function()
	if not a.PSFWatcherStarted then
		s.RegisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED", function (event, ...)
			local unit, _, _, _, spellID = ...
			if unit == "player" then
				if (spellID == 56641--[[Steady Shot]] or spellID == 77767--[[Cobra Shot]]) and not a.BuffPreSteadyFocus then
					a.BuffPreSteadyFocus = true
				elseif a.BuffPreSteadyFocus and tContains(ResetSteadyFocus, spellID)  then
					a.BuffPreSteadyFocus = false
				end
			end
		end)
		a.PSFWatcherStarted = true
	end
end

a.hasSteadyFocus = function()
	local TalentName = s.SpellName(177667--[[Steady Focus]], 1)
	local id, name, texture, selected = GetTalentInfo(4, 1, GetActiveSpecGroup())
	return TalentName == name and ( selected and 1 )
end

a.checkPSFWatcher = function (event, ...)
	if a.hasSteadyFocus() then
		if not a.PSFWatcherStarted then
			a.startPreSteadyFocus()
		end
	else
		if a.PSFWatcherStarted then
			a.stopPreSteadyFocus()
		end
	end
end

s.RegisterModuleEvent(AddonName, "ACTIVE_TALENT_GROUP_CHANGED", a.checkPSFWatcher)
s.RegisterModuleEvent(AddonName, "PLAYER_ENTERING_WORLD", a.checkPSFWatcher)
s.RegisterModuleEvent(AddonName, "PLAYER_SPECIALIZATION_CHANGED", a.checkPSFWatcher)
s.RegisterModuleEvent(AddonName, "ADDON_LOADED", function(event, arg)
	if arg == AddonName then
		if a.hasSteadyFocus() and not a.PSFWatcherStarted then
			a.startPreSteadyFocus()
		end
	end
end)

-- Aimed Shot is better than Arcane Shot with extreme values of haste
-- following are some rudimentary functions to make the module
-- aware of that. May be better to scan for an actual haste breakpoint.
local HasteCDs = {
	2825--[[Bloodlust]],
	3045--[[Rapid Fire]],
	26297--[[Berserking]],
	32182--[[Heroism]],
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	90355--[[Ancient Hysteria]],
	146555--[[Drums of Rage]],
	160452--[[Netherwinds]],
}
a.Hasted = function()
	return s.Buff(HasteCDs, "player", nil, false, false, true)
end

-- Detect raid-wide haste buffs, so we know when to
-- use potions
local BloodlustSpells = {
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	80353--[[Time Warp]],
	90355--[[Ancient Hysteria]],
	146555--[[Drums of Rage]],
	160452--[[Netherwinds]],
}
a.Bloodlust = function()
	return s.Buff(BloodlustSpells, "player", nil, false, false, true)
end

a.PetAssistActive = function()
    local name, active, exists
    local index = 10
    repeat
        name,_,_,_,active,_,_,exists = GetPetActionInfo(index)
        index = index - 1
        if name == "PET_MODE_ASSIST" then
			return active
        end
    until not exists
    return false
end

local regenRate
a.focusRegeneration = function()
	_, regenRate = GetPowerRegen()
	regenRate = format("%.2f", regenRate)
	return tonumber(regenRate)
end